@@ -5,9 +5,10 @@ import tempfile
 import time
 
 import pytest
+
 from PIL import Image, PdfParser
 
-from .helper import hopper
+from .helper import hopper, mark_if_feature_version
 
 
 def helper_save_as_pdf(tmp_path, mode, **kwargs):
@@ -29,7 +30,7 @@ def helper_save_as_pdf(tmp_path, mode, **kwargs):
     with open(outfile, "rb") as fp:
         contents = fp.read()
     size = tuple(
-        int(d) for d in contents.split(b"/MediaBox [ 0 0 ")[1].split(b"]")[0].split()
+        float(d) for d in contents.split(b"/MediaBox [ 0 0 ")[1].split(b"]")[0].split()
     )
     assert im.size == size
 
@@ -41,7 +42,8 @@ def test_monochrome(tmp_path):
     mode = "1"
 
     # Act / Assert
-    helper_save_as_pdf(tmp_path, mode)
+    outfile = helper_save_as_pdf(tmp_path, mode)
+    assert os.path.getsize(outfile) < 15000
 
 
 def test_greyscale(tmp_path):
@@ -84,6 +86,30 @@ def test_unsupported_mode(tmp_path):
         im.save(outfile)
 
 
+def test_resolution(tmp_path):
+    im = hopper()
+
+    outfile = str(tmp_path / "temp.pdf")
+    im.save(outfile, resolution=150)
+
+    with open(outfile, "rb") as fp:
+        contents = fp.read()
+
+    size = tuple(
+        float(d)
+        for d in contents.split(b"stream\nq ")[1].split(b" 0 0 cm")[0].split(b" 0 0 ")
+    )
+    assert size == (61.44, 61.44)
+
+    size = tuple(
+        float(d) for d in contents.split(b"/MediaBox [ 0 0 ")[1].split(b"]")[0].split()
+    )
+    assert size == (61.44, 61.44)
+
+
+@mark_if_feature_version(
+    pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
+)
 def test_save_all(tmp_path):
     # Single frame image
     helper_save_as_pdf(tmp_path, "RGB", save_all=True)
@@ -172,7 +198,7 @@ def test_pdf_open(tmp_path):
 def test_pdf_append_fails_on_nonexistent_file():
     im = hopper("RGB")
     with tempfile.TemporaryDirectory() as temp_dir:
-        with pytest.raises(IOError):
+        with pytest.raises(OSError):
             im.save(os.path.join(temp_dir, "nonexistent.pdf"), append=True)
 
 
@@ -284,3 +310,14 @@ def test_pdf_append_to_bytesio():
     f = io.BytesIO(f.getvalue())
     im.save(f, format="PDF", append=True)
     assert len(f.getvalue()) > initial_size
+
+
+@pytest.mark.timeout(1)
+@pytest.mark.parametrize("newline", (b"\r", b"\n"))
+def test_redos(newline):
+    malicious = b" trailer<<>>" + newline * 3456
+
+    # This particular exception isn't relevant here.
+    # The important thing is it doesn't timeout, cause a ReDoS (CVE-2021-25292).
+    with pytest.raises(PdfParser.PdfFormatError):
+        PdfParser.PdfParser(buf=malicious)
